%global with_debug 1

%{!?_with_bootstrap: %global bootstrap 0}

%{?!_pkgdocdir:%global _pkgdocdir %{_docdir}/%{name}-%{version}}

# ARM builds currently break on the Debug builds, so we'll just
# build the standard runtime until that gets sorted out.
%ifarch %{arm} aarch64 %{power64}
%global with_debug 0
%endif

# == Node.js Version ==
# Note: Mageia should only ship LTS versions of Node.js (currently expected
# to be major versions with even numbers). The odd-numbered versions are new
# feature releases that are only supported for nine months, which is shorter
# than a Mageia release lifecycle.
%global nodejs_epoch 1
%global nodejs_major 6
%global nodejs_minor 17
%global nodejs_patch 1
%global nodejs_abi %{nodejs_major}.%{nodejs_minor}
%global nodejs_version %{nodejs_major}.%{nodejs_minor}.%{nodejs_patch}
%global nodejs_release 8

# == Bundled Dependency Versions ==
# v8 - from deps/v8/include/v8-version.h
%global v8_major 5
%global v8_minor 1
%global v8_build 281
%global v8_patch 111
# V8 presently breaks ABI at least every x.y release while never bumping SONAME
%global v8_abi %{v8_major}.%{v8_minor}
%global v8_version %{v8_major}.%{v8_minor}.%{v8_build}.%{v8_patch}

# c-ares - from deps/cares/include/ares_version.h
%global c_ares_major 1
%global c_ares_minor 10
%global c_ares_patch 1
%global c_ares_version %{c_ares_major}.%{c_ares_minor}.%{c_ares_patch}

# http-parser - from deps/http_parser/http_parser.h
%global http_parser_major 2
%global http_parser_minor 8
%global http_parser_patch 0
%global http_parser_version %{http_parser_major}.%{http_parser_minor}.%{http_parser_patch}

# punycode - from lib/punycode.js
# Note: this was merged into the mainline since 0.6.x
# Note: this will be unmerged in v7 or v8
%global punycode_major 2
%global punycode_minor 0
%global punycode_patch 0
%global punycode_version %{punycode_major}.%{punycode_minor}.%{punycode_patch}

# npm - from deps/npm/package.json
%global npm_epoch 1
%global npm_major 3
%global npm_minor 10
%global npm_patch 10
%global npm_version %{npm_major}.%{npm_minor}.%{npm_patch}

# Filter out the NPM bundled dependencies so we aren't providing them
%global __provides_exclude_from ^%{_prefix}/lib/node_modules/npm/.*$
%global __requires_exclude_from ^%{_prefix}/lib/node_modules/npm/.*$


Name: nodejs
Epoch: %{nodejs_epoch}
Version: %{nodejs_version}
Release: %mkrel %{nodejs_release}
Summary: JavaScript runtime
License: MIT and ASL 2.0 and ISC and BSD
Group: Development/Other
URL: http://nodejs.org/

# nodejs bundles openssl, but we use the system version in Mageia
# because openssl contains prohibited code, we remove openssl completely from
# the tarball, using the script in Source100
Source0: node-v%{nodejs_version}-stripped.tar.gz
Source100: %{name}-tarball.sh

# The native module Requires generator remains in the nodejs SRPM, so it knows
# the nodejs and v8 versions.  The remainder has migrated to the
# nodejs-packaging SRPM.
Source7: nodejs_native.attr

# Disable running gyp on bundled deps we don't use
Patch1: 0001-Disable-running-gyp-files-for-bundled-deps.patch

BuildRequires: python-devel
BuildRequires: libuv-devel
Requires: libuv 
BuildRequires: libicu-devel
BuildRequires: zlib-devel
BuildRequires: gcc >= 5.5.0
BuildRequires: gcc-c++ >= 5.5.0

%if ! 0%{?bootstrap}
BuildRequires: systemtap
BuildRequires: http-parser-devel
%else
Provides: bundled(http-parser) = %{http_parser_version}
%endif

BuildRequires: openssl-devel

#we need ABI virtual provides where SONAMEs aren't enough/not present so deps
#break when binary compatibility is broken
Provides: nodejs(abi) = %{nodejs_abi}
Provides: nodejs(abi%{nodejs_major}) = %{nodejs_abi}
Provides: nodejs(v8-abi) = %{v8_abi}
Provides: nodejs(v8-abi%{v8_major}) = %{v8_abi}

#this corresponds to the "engine" requirement in package.json
Provides: nodejs(engine) = %{nodejs_version}

# Node.js currently has a conflict with the 'node' package in Mageia
# The ham-radio group has agreed to rename their binary for us, but
# in the meantime, we're setting an explicit Conflicts: here
Conflicts: node <= 0.3.2-12

# The punycode module was absorbed into the standard library in v0.6.
# It still exists as a seperate package for the benefit of users of older
# versions.  Since we've never shipped anything older than v0.10 in Mageia,
# we don't need the seperate nodejs-punycode package, so we Provide it here so
# dependent packages don't need to override the dependency generator.
# See also: RHBZ#11511811
# UPDATE: punycode will be deprecated and so we should unbundle it in Node v8
# and use upstream module instead
# https://github.com/nodejs/node/commit/29e49fc286080215031a81effbd59eac092fff2f
Provides: nodejs-punycode = %{punycode_version}
Provides: npm(punycode) = %{punycode_version}


# Node.js has forked c-ares from upstream in an incompatible way, so we need
# to carry the bundled version internally.
# See https://github.com/nodejs/node/commit/766d063e0578c0f7758c3a965c971763f43fec85
Provides: bundled(c-ares) = %{c_ares_version}

# Node.js is closely tied to the version of v8 that is used with it. It makes
# sense to use the bundled version because upstream consistently breaks ABI
# even in point releases. Node.js upstream has now removed the ability to build
# against a shared system version entirely.
# See https://github.com/nodejs/node/commit/d726a177ed59c37cf5306983ed00ecd858cfbbef
Provides: bundled(v8) = %{v8_version}

# Make sure we keep NPM up to date when we update Node.js
Requires: npm = %{npm_epoch}:%{npm_version}


%description
Node.js is a platform built on Chrome's JavaScript runtime
for easily building fast, scalable network applications.
Node.js uses an event-driven, non-blocking I/O model that
makes it lightweight and efficient, perfect for data-intensive
real-time applications that run across distributed devices.

%package devel
Summary: JavaScript runtime - development headers
Group: Development/Other
Requires: %{name}%{?_isa} = %{epoch}:%{nodejs_version}
Requires: libuv-devel
Requires: openssl-devel
Requires: zlib-devel
Requires: nodejs-packaging
%if ! 0%{?bootstrap}
Requires: http-parser-devel
%endif

%description devel
Development headers for the Node.js JavaScript runtime.

%package -n npm
Summary: Node.js Package Manager
Epoch: %{npm_epoch}
Version: %{npm_version}
# We used to ship npm separately, but it is so tightly integrated with Node.js
# (and expected to be present on all Node.js systems) that we ship it bundled
# now.
Obsoletes: npm < 0:3.5.4-6
Provides: npm = %{npm_epoch}:%{npm_version}
Requires: nodejs = %{nodejs_epoch}:%{nodejs_version}-%{release}

# Do not add epoch to the virtual NPM provides or it will break
# the automatic dependency-generation script.
Provides: npm(npm) = %{npm_version}

%description -n npm
npm is a package manager for node.js. You can use it to install and publish
your node programs. It manages dependencies and does other cool stuff.

%package docs
Summary: Node.js API documentation
Group: Documentation
BuildArch: noarch

# We don't require that the main package be installed to
# use the docs, but if it is installed, make sure the
# version always matches
Conflicts: %{name} > %{nodejs_epoch}:%{nodejs_version}-%{release}
Conflicts: %{name} < %{nodejs_epoch}:%{nodejs_version}-%{release}

%description docs
The API documentation for the Node.js JavaScript runtime.


%prep
%autosetup -p1 -n node-v%{nodejs_version}

rm -rf deps/icu-small \
       deps/uv \
       deps/zlib

%build
# build with debugging symbols and add defines from libuv (#892601)
# Node's v8 breaks with GCC 6 because of incorrect usage of methods on
# NULL objects. We need to pass -fno-delete-null-pointer-checks
export CFLAGS='%{optflags} -g \
               -D_LARGEFILE_SOURCE \
               -D_FILE_OFFSET_BITS=64 \
               -DZLIB_CONST \
               -fno-delete-null-pointer-checks'
export CXXFLAGS='%{optflags} -g \
                 -D_LARGEFILE_SOURCE \
                 -D_FILE_OFFSET_BITS=64 \
                 -DZLIB_CONST \
                 -fno-delete-null-pointer-checks'

# Explicit new lines in C(XX)FLAGS can break naive build scripts
export CFLAGS="$(echo ${CFLAGS} | tr '\n\\' '  ')"
export CXXFLAGS="$(echo ${CXXFLAGS} | tr '\n\\' '  ')"

%if ! 0%{?bootstrap}
./configure --prefix=%{_prefix} \
           --shared-openssl \
           --shared-zlib \
           --shared-libuv \
           --shared-http-parser \
           --with-dtrace \
           --with-intl=system-icu \
           --openssl-use-def-ca-store
%else
./configure --prefix=%{_prefix} \
           --shared-openssl \
           --shared-zlib \
           --shared-libuv \
           --without-dtrace \
           --with-intl=system-icu \
           --openssl-use-def-ca-store
%endif

%if %{?with_debug} == 1
# Setting BUILDTYPE=Debug builds both release and debug binaries
make BUILDTYPE=Debug %{?_smp_mflags}
%else
make BUILDTYPE=Release %{?_smp_mflags}
%endif


%install
rm -rf %{buildroot}

./tools/install.py install %{buildroot} %{_prefix}

# Set the binary permissions properly
chmod 0755 %{buildroot}/%{_bindir}/node

%if %{?with_debug} == 1
# Install the debug binary and set its permissions
install -Dpm0755 out/Debug/node %{buildroot}/%{_bindir}/node_g
%endif

# own the sitelib directory
mkdir -p %{buildroot}%{_prefix}/lib/node_modules

# ensure Requires are added to every native module that match the Provides from
# the nodejs build in the buildroot
install -Dpm0644 %{SOURCE7} %{buildroot}%{_rpmconfigdir}/fileattrs/nodejs_native.attr
cat << EOF > %{buildroot}%{_rpmconfigdir}/nodejs_native.req
#!/bin/sh
echo 'nodejs(abi%{nodejs_major}) >= %nodejs_abi'
echo 'nodejs(v8-abi%{v8_major}) >= %v8_abi'
EOF
chmod 0755 %{buildroot}%{_rpmconfigdir}/nodejs_native.req

#install documentation
mkdir -p %{buildroot}%{_pkgdocdir}/html
cp -pr doc/* %{buildroot}%{_pkgdocdir}/html
rm -f %{buildroot}%{_pkgdocdir}/html/nodejs.1

#node-gyp needs common.gypi too
mkdir -p %{buildroot}%{_datadir}/node
cp -p common.gypi %{buildroot}%{_datadir}/node

# Install the GDB init tool into the documentation directory
mv %{buildroot}/%{_datadir}/doc/node/gdbinit %{buildroot}/%{_pkgdocdir}/gdbinit

# Since the old version of NPM was unbundled, there are a lot of symlinks in
# it's node_modules directory. We need to keep these as symlinks to ensure we
# can backtrack on this if we decide to.

# Rename the npm node_modules directory to node_modules.bundled
mv %{buildroot}/%{_prefix}/lib/node_modules/npm/node_modules \
   %{buildroot}/%{_prefix}/lib/node_modules/npm/node_modules.bundled

# Recreate all the symlinks
mkdir -p %{buildroot}/%{_prefix}/lib/node_modules/npm/node_modules
FILES=%{buildroot}/%{_prefix}/lib/node_modules/npm/node_modules.bundled/*
for f in $FILES
do
  module=`basename $f`
  ln -s ../node_modules.bundled/$module %{buildroot}%{_prefix}/lib/node_modules/npm/node_modules/$module
done

# install NPM docs to mandir
mkdir -p %{buildroot}%{_mandir} \
         %{buildroot}%{_pkgdocdir}/npm

cp -pr deps/npm/man/* %{buildroot}%{_mandir}/
rm -rf %{buildroot}%{_prefix}/lib/node_modules/npm/man
ln -sf %{_mandir}  %{buildroot}%{_prefix}/lib/node_modules/npm/man

# Install Markdown and HTML documentation to %{_pkgdocdir}
cp -pr deps/npm/html deps/npm/doc %{buildroot}%{_pkgdocdir}/npm/
rm -rf %{buildroot}%{_prefix}/lib/node_modules/npm/html \
       %{buildroot}%{_prefix}/lib/node_modules/npm/doc

ln -sf %{_pkgdocdir} %{buildroot}%{_prefix}/lib/node_modules/npm/html
ln -sf %{_pkgdocdir}/npm/html %{buildroot}%{_prefix}/lib/node_modules/npm/doc


# Node tries to install some python files into a documentation directory
# (and not the proper one). Remove them for now until we figure out what to
# do with them.
rm -f %{buildroot}/%{_defaultdocdir}/node/lldb_commands.py \
      %{buildroot}/%{_defaultdocdir}/node/lldbinit

%check
# Fail the build if the versions don't match
%{buildroot}/%{_bindir}/node -e "require('assert').equal(process.versions.node, '%{nodejs_version}')"
%{buildroot}/%{_bindir}/node -e "require('assert').equal(process.versions.v8, '%{v8_version}')"
%{buildroot}/%{_bindir}/node -e "require('assert').equal(process.versions.ares.replace(/-DEV$/, ''), '%{c_ares_version}')"

# Ensure we have punycode and that the version matches
%{buildroot}/%{_bindir}/node -e "require(\"assert\").equal(require(\"punycode\").version, '%{punycode_version}')"

# Ensure we have npm and that the version matches
NODE_PATH=%{buildroot}%{_prefix}/lib/node_modules %{buildroot}/%{_bindir}/node -e "require(\"assert\").equal(require(\"npm\").version, '%{npm_version}')"

%files
%{_bindir}/node
%dir %{_prefix}/lib/node_modules
%dir %{_datadir}/node
%dir %{_datadir}/systemtap
%dir %{_datadir}/systemtap/tapset
%{_datadir}/systemtap/tapset/node.stp

%if ! 0%{?bootstrap}
%dir %{_usr}/lib/dtrace
%{_usr}/lib/dtrace/node.d
%endif

%{_rpmconfigdir}/fileattrs/nodejs_native.attr
%{_rpmconfigdir}/nodejs_native.req
%license LICENSE
%doc AUTHORS CHANGELOG.md COLLABORATOR_GUIDE.md GOVERNANCE.md README.md
%doc %{_mandir}/man1/node.1*


%files devel
%if %{?with_debug} == 1
%{_bindir}/node_g
%endif
%{_includedir}/node
%{_datadir}/node/common.gypi
%{_pkgdocdir}/gdbinit


%files -n npm
%{_bindir}/npm
%{_prefix}/lib/node_modules/npm
%ghost %{_sysconfdir}/npmrc
%ghost %{_sysconfdir}/npmignore
%doc %{_mandir}/man*/npm*
%doc %{_mandir}/man5/package.json.5*
%doc %{_mandir}/man7/removing-npm.7*
%doc %{_mandir}/man7/semver.7*


%files docs
%dir %{_pkgdocdir}
%{_pkgdocdir}/html
%{_pkgdocdir}/npm/html
%{_pkgdocdir}/npm/doc



%changelog
* Sun Jun 09 2019 mokraemer <mokraemer> 1:6.17.1-8.mga6
+ Revision: 1399721
- wired docs comment
- Fix devel dependencies
+ kekepower <kekepower>
- Second try at fixing installation issue
- Hopefully fix the issue of not being able to install
- Fix bogus release tag for npm
- Fix npm release version
- Update to version 6.17.1
- Fixes several security issues (mga#21330)
+ neoclust <neoclust>
- New LTS version 6.11.2
- Try to rebuild to make it instalable
- Fix release to install smoothly
- Rebuild with fixed release
- Do not use %%mkrel
- Rebuild with fixed deps
- Relax requires
- New version v6.11.1

* Fri May 05 2017 neoclust <neoclust> 1:6.10.3-2.mga6
+ Revision: 1099146
- Rebuild because of missing package

* Thu May 04 2017 neoclust <neoclust> 1:6.10.3-1.mga6
+ Revision: 1099118
- New version 6.10.3 (LTS)

* Thu May 04 2017 neoclust <neoclust> 1:6.10.2-1.mga6
+ Revision: 1099101
- New version v6.10.2 ( LTS )
- New version 6.9.2

* Wed Mar 15 2017 mrambo3501 <mrambo3501> 1:6.9.1-3.mga6
+ Revision: 1092754
- Rebuild for icu-58.2

* Sun Dec 11 2016 ngompa <ngompa> 1:6.9.1-2.mga6
+ Revision: 1074259
- Convert %%pretrans to Lua

* Sat Nov 12 2016 shlomif <shlomif> 1:6.9.1-1.mga6
+ Revision: 1066886
- New stable version v6.9.1.

* Thu Sep 08 2016 luigiwalser <luigiwalser> 1:4.5.0-1.mga6
+ Revision: 1050787
- 4.5.0 (sync with f24)
- split npm into a subpackage
- build Node.js with internationalization support

* Wed May 25 2016 neoclust <neoclust> 1:4.4.4-3.mga6
+ Revision: 1018324
- Rebuild for ARM

* Tue May 17 2016 tv <tv> 1:4.4.4-2.mga6
+ Revision: 1016533
- fix upgrade

* Tue May 17 2016 neoclust <neoclust> 1:4.4.4-1.mga6
+ Revision: 1016407
- Update to latest LTS security release (4.4.4)
  https://github.com/nodejs/node/blob/v4.4.4/CHANGELOG.md
  https://lwn.net/Articles/686158/

* Mon May 16 2016 neoclust <neoclust> 6.1.0-5.mga6
+ Revision: 1016185
- Add conflict to ease upgrade

* Mon May 16 2016 neoclust <neoclust> 6.1.0-4.mga6
+ Revision: 1016066
- Remove useless epoch

* Mon May 16 2016 joequant <joequant> 1:6.1.0-3.mga6
+ Revision: 1015965
- change requires from ca-certificates -> rootcerts

* Mon May 16 2016 neoclust <neoclust> 1:6.1.0-2.mga6
+ Revision: 1015842
- Fix ExclusiveArch
- New version 6.1.0

* Mon Apr 25 2016 joequant <joequant> 5.11.0-1.mga6
+ Revision: 1005991
- update to 5.11.0

* Wed Mar 16 2016 tv <tv> 5.6.0-3.mga6
+ Revision: 992158
- resync patches with FC (thus fixing build)

* Wed Mar 02 2016 umeabot <umeabot> 5.6.0-2.mga6
+ Revision: 983804
- Rebuild for openssl

* Mon Feb 15 2016 joequant <joequant> 5.6.0-1.mga6
+ Revision: 961716
- update to 5.6.0 - fixes 17779

* Fri Jan 15 2016 joequant <joequant> 5.4.1-1.mga6
+ Revision: 923221
- upgrade to 5.4.1

* Sat Dec 19 2015 joequant <joequant> 5.3.0-2.mga6
+ Revision: 912183
- disable debug builds for arm

* Sat Dec 19 2015 joequant <joequant> 5.3.0-1.mga6
+ Revision: 912136
- upgrade to 5.3.0

* Sat Dec 19 2015 joequant <joequant> 4.2.3-2.mga6
+ Revision: 912125
- change category
- fix category
- upgrade to 4.2.3

* Thu Jul 30 2015 joequant <joequant> 0.10.40-1.mga6
+ Revision: 859093
- update to 0.10.40 and include node-gyp

* Sun May 03 2015 joequant <joequant> 0.10.38-1.mga5
+ Revision: 821164
- update to 0.10.38 to fix security issues in bug 15405

* Mon Oct 27 2014 luigiwalser <luigiwalser> 0.10.33-1.mga5
+ Revision: 793974
- 0.10.33

* Wed Oct 15 2014 umeabot <umeabot> 0.10.30-6.mga5
+ Revision: 750309
- Second Mageia 5 Mass Rebuild

* Sat Sep 27 2014 tv <tv> 0.10.30-5.mga5
+ Revision: 726006
- add native deps support from FC

* Fri Sep 26 2014 tv <tv> 0.10.30-4.mga5
+ Revision: 725139
- rebuild for bogus file deps

* Wed Sep 17 2014 joequant <joequant> 0.10.30-3.mga5
+ Revision: 693134
- fix deps for nodejs
+ umeabot <umeabot>
- Mageia 5 Mass Rebuild

* Sat Aug 16 2014 dams <dams> 0.10.30-1.mga5
+ Revision: 664405
- new version 0.10.30

* Wed Jul 23 2014 dams <dams> 0.10.29-1.mga5
+ Revision: 655770
- new version 0.10.29

* Fri May 02 2014 dams <dams> 0.10.28-1.mga5
+ Revision: 619739
- new version 0.10.28

* Mon Feb 24 2014 dams <dams> 0.10.26-1.mga5
+ Revision: 596210
- new version 0.10.26

* Tue Feb 04 2014 dams <dams> 0.10.25-1.mga5
+ Revision: 580750
- new version 0.10.25

* Mon Dec 23 2013 dams <dams> 0.10.24-1.mga4
+ Revision: 560066
- new version 0.10.24

* Wed Nov 20 2013 dams <dams> 0.10.22-1.mga4
+ Revision: 552078
- new version 0.10.22

* Wed Oct 23 2013 dams <dams> 0.10.21-1.mga4
+ Revision: 546474
- new version 0.10.21 (DoS Vulnerability)

* Mon Oct 21 2013 umeabot <umeabot> 0.10.20-2.mga4
+ Revision: 538513
- Mageia 4 Mass Rebuild

* Tue Oct 01 2013 dams <dams> 0.10.20-1.mga4
+ Revision: 489822
- new version 0.10.20
- new version 0.10.20

* Wed Sep 25 2013 dams <dams> 0.10.19-1.mga4
+ Revision: 485929
- new version 0.10.19

* Thu Sep 05 2013 dams <dams> 0.10.18-1.mga4
+ Revision: 475233
- new version 0.10.18

* Mon Aug 26 2013 dams <dams> 0.10.17-1.mga4
+ Revision: 471919
- new version 0.10.17

* Sat Jul 27 2013 dams <dams> 0.10.15-1.mga4
+ Revision: 458753
- new version 0.10.15

* Mon Jul 15 2013 dams <dams> 0.10.13-1.mga4
+ Revision: 454553
- new version 0.10.13

* Wed Jun 19 2013 dams <dams> 0.10.12-1.mga4
+ Revision: 445084
- new version 0.10.12

* Thu Jun 13 2013 dams <dams> 0.10.11-1.mga4
+ Revision: 442902
- new version 0.10.11

* Sat Jun 08 2013 dams <dams> 0.10.10-1.mga4
+ Revision: 440640
- new version 0.10.10

* Fri May 31 2013 dams <dams> 0.10.9-1.mga4
+ Revision: 433721
- new version 0.10.9

* Mon May 27 2013 dams <dams> 0.10.8-1.mga4
+ Revision: 428717
- new version 0.10.8

* Thu May 23 2013 dams <dams> 0.10.7-1.mga4
+ Revision: 425004
- new version 0.10.7

* Thu Apr 11 2013 dams <dams> 0.10.3-2.mga3
+ Revision: 409688
- now use bundled v8 as requested by Luigi12 to avoid using buggy v8 package

* Sat Apr 06 2013 dams <dams> 0.10.3-1.mga3
+ Revision: 408511
- new version 0.10.3

* Tue Apr 02 2013 dams <dams> 0.10.2-1.mga3
+ Revision: 407207
- new version 0.10.2

* Fri Mar 29 2013 dams <dams> 0.10.1-1.mga3
+ Revision: 406472
- disable doc for now
- new stable version 0.10.1
- clean and update BR
- add a 'docs' subpackage
- make sure we are not using bundled deps

* Sat Feb 09 2013 fwang <fwang> 0.9.9-1.mga3
+ Revision: 397647
- use shared cares
- update file list
- new version 0.9.9
- rebuild for new v8

* Sat Feb 09 2013 fwang <fwang> 0.9.0-4.mga3
+ Revision: 397503
- rebuild for new v8
+ umeabot <umeabot>
- Mass Rebuild - https://wiki.mageia.org/en/Feature:Mageia3MassRebuild

* Mon Sep 24 2012 fedya <fedya> 0.9.0-2.mga3
+ Revision: 297232
- Provides: npm see #820

* Mon Jul 30 2012 shlomif <shlomif> 0.9.0-1.mga3
+ Revision: 276171
- New version 0.9.0 thanks to fedya.
+ dmorgan <dmorgan>
- New version 0.8.1

* Wed Jan 11 2012 fwang <fwang> 0.4.12-1.mga2
+ Revision: 195003
- new version 0.4.12
+ misc <misc>
- new version
+ shikamaru <shikamaru>
- imported package nodejs

